# Dynamically Generated
@builtin "number.ne"
@builtin "string.ne"
@builtin "whitespace.ne"
@{%
    function compact(v) {
        if (Array.isArray(v)) {
            return v.reduce((a, v) => (v === null || v === undefined || (v && v.length === 0) ) ? a : (a.push(v), a), []);
        } else {
            return v;
        }
    }

    function flatten(v) {
        let r;
        if (Array.isArray(v)) {
            r = v.reduce((a,v) => (a.push(...((v && Array.isArray(v)) ? flatten(v) : [v])), a), []);
        } else {
            r = v;
        }
        r = compact(r);
        return r;
        }
%}


rule -> rule0 rule1 rule2 rule3 rule4 rule5 rule6 rule7 rule8 
{% ([ rule0,rule1,rule2,rule3,rule4,rule5,rule6,rule7,rule8 ]) => {
    return {
        $class: "org.accordproject.demandforecast.TemplateModel",
        
        purchaser : rule3,
        supplier : rule5,
        effectiveDate : rule1,
        minimumPercentage : rule7
    };
}
%}

ROOT -> rule0 
{% ([ rule0 ]) => {
    return {
        
        
    };
}
%}



rule0 -> "On or before the " 


rule1 -> DateTime {% id %} # effectiveDate 


rule2 -> ", and thereafter on or before the last day of each calendar quarter, the Purchaser " 


rule3 -> String {% id %} # purchaser 


rule4 -> " shall send the Supplier " 


rule5 -> String {% id %} # supplier 


rule6 -> " a Supply Forecast. The Supply Forecast will be a binding commitment on the Purchaser’s behalf to purchase during the first quarter included in each Supply Forecast at least " 


rule7 -> Double {% id %} # minimumPercentage 


rule8 -> "% of the quantity of the listed Product.\n" 


ContractState -> String  __  String 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ContractState",
        stateId : data[0],
        contract : data[2]
    };
}
%}


ContractModel -> String 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ContractModel",
        contractId : data[0]
    };
}
%}


ClauseModel -> String 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ClauseModel",
        clauseId : data[0]
    };
}
%}


Request -> String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Request",
        transactionId : data[0],
        timestamp : data[2]
    };
}
%}


Response -> String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Response",
        transactionId : data[0],
        timestamp : data[2]
    };
}
%}


ErrorResponse -> String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.ErrorResponse",
        transactionId : data[0],
        timestamp : data[2]
    };
}
%}


Payload -> ContractModel  __  Request  __  ContractState 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Payload",
        contract : data[0],
        request : data[2],
        state : data[4]
    };
}
%}


Success -> Response  __  ContractState  __  Event:+ 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Success",
        response : data[0],
        state : data[2],
        emit : data[4]
    };
}
%}


Failure -> ErrorResponse 

{% ( data ) => {
    return {
        $class: "org.accordproject.common.Failure",
        error : data[0]
    };
}
%}


ForecastRequest -> Integer  __  String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.demandforecast.ForecastRequest",
        supplyForecast : data[0],
        transactionId : data[2],
        timestamp : data[4]
    };
}
%}


BindingResponse -> Double  __  Integer  __  Integer  __  String  __  DateTime 

{% ( data ) => {
    return {
        $class: "org.accordproject.demandforecast.BindingResponse",
        requiredPurchase : data[0],
        year : data[2],
        quarter : data[4],
        transactionId : data[6],
        timestamp : data[8]
    };
}
%}



# Basic types
NUMBER -> [0-9] 
{% (d) => {return parseInt(d[0]);}%}

DOUBLE_NUMBER -> NUMBER NUMBER
{% (d) => {return '' + d[0] + d[1]}%}

MONTH -> DOUBLE_NUMBER
DAY -> DOUBLE_NUMBER
YEAR -> DOUBLE_NUMBER DOUBLE_NUMBER
{% (d) => {return '' + d[0] + d[1]}%}

DATE -> MONTH "/" DAY "/" YEAR
{% (d) => {return '' + d[4] + '-' + d[0] + '-' + d[2]}%}

Word -> [\S]:*
{% (d) => {return d[0].join('');}%}

BRACKET_PHRASE -> "[" Word (__ Word):* "]" {% ((d) => {return d[1] + ' ' + flatten(d[2]).join(" ");}) %}

String -> dqstring {% id %}
Double -> decimal {% id %}
Integer -> int {% id %}
Long -> int {% id %}
Boolean -> "true" {% id %} | "false" {% id %}
DateTime -> DATE  {% id %}